@@ -65,6 +65,14 @@ pub enum Value<'a> {
 
     Text(&'a str),
     Bytes(&'a [u8]),
+
+    #[cfg(feature = "use-chrono")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "use-chrono")))]
+    DateTime(chrono::DateTime<chrono::Utc>),
+
+    #[cfg(feature = "use-decimal")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "use-decimal")))]
+    Numeric(rust_decimal::Decimal),
 }
 
 crate::macros::gen_display!(Value<'_>);
@@ -83,6 +91,14 @@ pub enum Null<'a> {
     BigUInt(PhantomData<u64>),
     Text(PhantomData<&'a str>),
     Bytes(PhantomData<&'a [u8]>),
+
+    #[cfg(feature = "use-chrono")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "use-chrono")))]
+    DateTime(PhantomData<chrono::DateTime<chrono::Utc>>),
+
+    #[cfg(feature = "use-decimal")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "use-decimal")))]
+    Numeric(PhantomData<rust_decimal::Decimal>),
 }
 
 into_value!(
@@ -97,6 +113,16 @@ into_value!(
     u64 => BigUInt,
 );
 
+#[cfg(feature = "use-chrono")]
+into_value!(
+    chrono::DateTime<chrono::Utc> => DateTime,
+);
+
+#[cfg(feature = "use-decimal")]
+into_value!(
+    rust_decimal::Decimal => Numeric,
+);
+
 into_borrowed_value!(
     String => Text,
     Vec<u8> => Bytes,
